// Copyright 2019  github.com. All rights reserved.
// Use of this source code is governed by  github.com.
package v1

import (
	"context"
	"encoding/json"
	"errors"
	"fmt"
	"time"
	"tower-monitor/consts"
	"tower-monitor/model"
	"tower-monitor/pb"
	"tower-monitor/pb/v1"

	"github.com/jaryhe/gopkgs/logger"
	"go.uber.org/zap"
)

var towerPositionAlarmReason = map[int]string{
	0x0: "正常",
	0x1: "预警",
	0x2: "报警",
	0x4: "故障",
}
var towerMachinePositionAlarmReason = map[int]string{
	0: "正常",
	1: "故障",
}
var towerWeightAlarmReason = map[int]string{
	0x0: "正常",
	0x1: "重载",
	0x2: "超载",
	0x3: "违章",
	0x4: "故障",
}
var towerWindAlarmReason = map[int]string{
	0x0: "正常",
	0x1: "重载",
	0x2: "超载",
	0x3: "违章",
	0x4: "故障",
}

var towerAngleAlarmReason = map[int]string{
	0: "正常",
	2: "报警",
}
var towerMomentAlarmReason = map[int]string{
	0x0: "正常",
	0x1: "重载",
	0x2: "超载",
	0x3: "违章",
	0x4: "故障",
}

var towerArmDirection = map[byte]string{
	0: "前臂",
	1: "后臂",
}
var towerCollisionAlarmReason = map[int]string{
	0x0: "正常",
	0x1: "预警",
	0x2: "报警",
}

var towerForbiddenAlarmReason = map[int]string{
	0x0: "正常",
	0x1: "重载",
	0x2: "超载",
	0x3: "违章",
}

var towerObstaclefAlarmReason = map[int]string{
	0x0: "正常",
	0x1: "预警",
	0x2: "报警",
}

var towerRelayAlarmReason = map[int]string{
	0x0: "正常",
	0x1: "截断低速",
	0x2: "截断高速",
}

const (
	LeftPositionAlarmName         = "左限位"
	RightPositionAlarmName        = "右限位"
	FarPositionAlarmName          = "远限位"
	NearPositionAlarmName         = "近限位"
	HighPositionAlarmName         = "高限位"
	LowPositionAlarmName          = "低限位"
	LeftMachinePositionAlarmName  = "回转机械左限位"
	RightMachinePositionAlarmName = "回转机械右限位"
	FarMachinePositionAlarmName   = "变幅机械远限位"
	NearMachinePositionAlarmName  = "变幅机械近限位"
	HightMachinePositionAlarmName  = "高度机械限位"
	WeightMachinePositionAlarmName   = "起重机械限位"
	MomentMachinePositionAlarmName = "力矩机械限位"
	WeightAlarmName             = "起重"
	MomentAlarmName             = "力矩"
	WindAlarmName               = "风速"
	AngleAlarmName = "塔身倾角"
	LeftCollisionAlarmName      = "左碰撞"
	RightCollisionAlarmName     = "右碰撞"
	FarCollisionAlarmName       = "远碰撞"
	NearCollisionAlarmName      = "近碰撞"
	LowCollisionAlarmName       = "低碰撞"
	LeftForbiddenAlarmName      = "禁区碰撞左侧禁行"
	RightForbiddenAlarmName     = "禁区碰撞右侧禁行"
	LeftObstacleAlarmName       = "左障碍"
	RightObstacleAlarmName      = "右障碍"
	FarObstacleAlarmName        = "远障碍"
	NearObstacleAlarmName       = "近障碍"
	LowObstacleAlarmName        = "低障碍"
	LeftRelayAlarmName          = "继电输出左"
	RightRelayAlarmName         = "继电输出右"
	FarRelayAlarmName           = "继电输出远"
	NearRelayAlarmName          = "继电输出近"
	HighRelayAlarmName          = "继电输出高"
	LowRelayAlarmName           = "继电输出低"
)

var towerAlarmCodePre = map[string]string{
	LeftPositionAlarmName:         "T01",
	RightPositionAlarmName:        "T02",
	FarPositionAlarmName:          "T03",
	NearPositionAlarmName:         "T04",
	HighPositionAlarmName:         "T05",
	LowPositionAlarmName:          "T06",
	LeftMachinePositionAlarmName:  "T07",
	RightMachinePositionAlarmName: "T08",
	FarMachinePositionAlarmName:   "T09",
	NearMachinePositionAlarmName:  "T10",
	HightMachinePositionAlarmName:  "T11",
	WeightMachinePositionAlarmName:   "T12",
	MomentMachinePositionAlarmName:   "T13",
	WeightAlarmName:             "T14",
	MomentAlarmName:             "T15",
	WindAlarmName:               "T16",
	AngleAlarmName: "T17",
	LeftCollisionAlarmName:      "T18",
	RightCollisionAlarmName:     "T19",
	FarCollisionAlarmName:       "T20",
	NearCollisionAlarmName:      "T21",
	LowCollisionAlarmName:       "T22",
	LeftForbiddenAlarmName:      "T23",
	RightForbiddenAlarmName:     "T24",
	LeftObstacleAlarmName:       "T25",
	RightObstacleAlarmName:      "T26",
	FarObstacleAlarmName:        "T27",
	NearObstacleAlarmName:       "T28",
	LowObstacleAlarmName:        "T29",
	LeftRelayAlarmName:          "T30",
	RightRelayAlarmName:         "T31",
	FarRelayAlarmName:           "T32",
	NearRelayAlarmName:          "T33",
	HighRelayAlarmName:          "T34",
	LowRelayAlarmName:           "T35",
}

func handleAlarm(ctx context.Context, alram *v1.AlarmAddRequest, title string, reasonCode int, directionCode byte, reasonMap map[int]string, directionMap map[byte]string) error {

	// 前臂 后臂
	direction := ""
	if directionMap != nil {
		str, ok := directionMap[directionCode]
		if !ok {
			return errors.New(fmt.Sprintf("%s unknown direction %d", title, directionCode))
		}
		direction = str
	}
	reason, ok := reasonMap[reasonCode]
	if !ok {
		return errors.New(fmt.Sprintf("%s unknown reason %d", title, reasonCode))
	}

	alram.AlarmReason = direction + title + reason
	alram.AlarmCode = towerAlarmCodePre[title]
	alram.DeviceCode = consts.TowerDeviceCode
	// 添加告警
	_, err := pb.SmartAlarm.AlarmAdd(ctx, alram)
	if err != nil {
		reqByte, _ := json.Marshal(*alram)
		logger.Error("rpc",
			zap.String("call", "pb.SmartAlarm.AlarmAdd"),
			zap.String("args", string(reqByte)),
			zap.String("error", err.Error()))
		return err
	}
	return nil
}

func handleTowerAlarmReason(ctx context.Context, req *TowerRealtimeDataRequest, projectId int64, deviceName string) error {
	alram := &v1.AlarmAddRequest{
		Date:       time.Unix(req.Timestamp, 0).Format("2006-01-02 15:04:05"),
		ProjectId:  projectId,
		Sn:         req.Sn,
		DeviceName: deviceName,
		DeviceType: consts.TowerDeviceName,
	}

	// 限位告警
	if req.PositionLeftAlarm > 0 {
		handleAlarm(ctx, alram, LeftPositionAlarmName, req.PositionLeftAlarm, 0, towerPositionAlarmReason, nil)
	}
	if req.PositionRightAlarm > 0 {
		handleAlarm(ctx, alram, RightPositionAlarmName, req.PositionRightAlarm, 0, towerPositionAlarmReason, nil)
	}
	if req.PositionFarAlarm > 0 {
		handleAlarm(ctx, alram, FarPositionAlarmName, req.PositionFarAlarm, 0, towerPositionAlarmReason, nil)
	}
	if req.PositionNearAlarm > 0 {
		handleAlarm(ctx, alram, NearPositionAlarmName, req.PositionNearAlarm, 0, towerPositionAlarmReason, nil)
	}
	if req.PositionHighAlarm > 0 {
		handleAlarm(ctx, alram, HighPositionAlarmName, req.PositionHighAlarm, 0, towerPositionAlarmReason, nil)
	}
	if req.PositionLowAlarm > 0 {
		handleAlarm(ctx, alram, LowPositionAlarmName, req.PositionLowAlarm, 0, towerPositionAlarmReason, nil)
	}
	if req.MachineBackTurnLeftAlarm > 0 {
		handleAlarm(ctx, alram, LeftMachinePositionAlarmName, int(req.MachineBackTurnLeftAlarm), 0, towerMachinePositionAlarmReason, nil)
	}
	if req.MachineBackTurnRightAlarm > 0 {
		handleAlarm(ctx, alram, RightMachinePositionAlarmName, int(req.MachineBackTurnRightAlarm), 0, towerMachinePositionAlarmReason, nil)
	}
	if req.MachineScopeFarAlarm > 0 {
		handleAlarm(ctx, alram, FarMachinePositionAlarmName, int(req.MachineScopeFarAlarm), 0, towerMachinePositionAlarmReason, nil)
	}
	if req.MachineScopeNearAlarm > 0 {
		handleAlarm(ctx, alram, NearMachinePositionAlarmName, int(req.MachineScopeNearAlarm), 0, towerMachinePositionAlarmReason, nil)
	}
	if req.MachineHighAlarm > 0 {
		handleAlarm(ctx, alram, HightMachinePositionAlarmName, int(req.MachineHighAlarm), 0, towerMachinePositionAlarmReason, nil)
	}
	if req.MachineWeightAlarm > 0 {
		handleAlarm(ctx, alram, WeightMachinePositionAlarmName, int(req.MachineWeightAlarm), 0, towerMachinePositionAlarmReason, nil)
	}
	if req.MachineMomentAlarm > 0 {
		handleAlarm(ctx, alram, MomentMachinePositionAlarmName, int(req.MachineMomentAlarm), 0, towerMachinePositionAlarmReason, nil)
	}

	// 其他告警
	if req.WeightAlarm > 0 {
		handleAlarm(ctx, alram, WeightAlarmName, req.WeightAlarm, 0, towerWeightAlarmReason, nil)
	}
	if req.MomentAlarm > 0 {
		handleAlarm(ctx, alram, MomentAlarmName, req.MomentAlarm, 0, towerMomentAlarmReason, nil)
	}
	if req.WindSpeedAlarm > 0 {
		handleAlarm(ctx, alram, WindAlarmName, req.WindSpeedAlarm, 0, towerWindAlarmReason, nil)
	}
	if req.AngleAlarm > 0 {
		handleAlarm(ctx, alram, AngleAlarmName, req.AngleAlarm, 0, towerAngleAlarmReason, nil)
	}

	// 塔机碰撞告警
	if req.LeftCollisionAlarm > 0 {
		handleAlarm(ctx, alram, LeftCollisionAlarmName, req.LeftCollisionAlarm, req.CollisionDirection, towerCollisionAlarmReason, towerArmDirection)
	}

	if req.RightCollisionAlarm > 0 {
		handleAlarm(ctx, alram, RightCollisionAlarmName, req.RightCollisionAlarm, req.CollisionDirection, towerCollisionAlarmReason, towerArmDirection)
	}
	if req.FarCollisionAlarm > 0 {
		handleAlarm(ctx, alram, FarCollisionAlarmName, req.FarCollisionAlarm, req.CollisionDirection, towerCollisionAlarmReason, towerArmDirection)
	}
	if req.NearCollisionAlarm > 0 {
		handleAlarm(ctx, alram, NearCollisionAlarmName, req.NearCollisionAlarm, req.CollisionDirection, towerCollisionAlarmReason, towerArmDirection)
	}
	if req.LowCollisionAlarm > 0 {
		handleAlarm(ctx, alram, LowCollisionAlarmName, req.LowCollisionAlarm, req.CollisionDirection, towerCollisionAlarmReason, towerArmDirection)
	}

	// 禁行区碰撞报警
	if req.ForbiddenLeft > 0 {
		handleAlarm(ctx, alram, LeftForbiddenAlarmName, req.ForbiddenLeft, req.ForbiddenDirection, towerForbiddenAlarmReason, towerArmDirection)
	}
	if req.ForbiddenRight > 0 {
		handleAlarm(ctx, alram, RightForbiddenAlarmName, req.ForbiddenRight, req.ForbiddenDirection, towerForbiddenAlarmReason, towerArmDirection)
	}

	// 障碍物碰撞报警
	if req.ObstacleLeftCollisionAlarm > 0 {
		handleAlarm(ctx, alram, LeftObstacleAlarmName, req.ObstacleLeftCollisionAlarm, req.ObstacleCollisionDirection, towerObstaclefAlarmReason, towerArmDirection)
	}
	if req.ObstacleRightCollisionAlarm > 0 {
		handleAlarm(ctx, alram, RightObstacleAlarmName, req.ObstacleRightCollisionAlarm, req.ObstacleCollisionDirection, towerObstaclefAlarmReason, towerArmDirection)
	}
	if req.ObstacleFarCollisionAlarm > 0 {
		handleAlarm(ctx, alram, FarObstacleAlarmName, req.ObstacleFarCollisionAlarm, req.ObstacleCollisionDirection, towerObstaclefAlarmReason, towerArmDirection)
	}
	if req.ObstacleNearCollisionAlarm > 0 {
		handleAlarm(ctx, alram, NearObstacleAlarmName, req.ObstacleNearCollisionAlarm, req.ObstacleCollisionDirection, towerObstaclefAlarmReason, towerArmDirection)
	}
	if req.ObstacleLowCollisionAlarm > 0 {
		handleAlarm(ctx, alram, LowObstacleAlarmName, req.ObstacleLowCollisionAlarm, req.ObstacleCollisionDirection, towerObstaclefAlarmReason, towerArmDirection)
	}

	// 继电输出
	if req.RelayLeft > 0 {
		handleAlarm(ctx, alram, LeftRelayAlarmName, req.RelayLeft, 0, towerRelayAlarmReason, nil)
	}
	if req.RelayRight > 0 {
		handleAlarm(ctx, alram, RightRelayAlarmName, req.RelayRight, 0, towerRelayAlarmReason, nil)
	}
	if req.RelayFar > 0 {
		handleAlarm(ctx, alram, FarRelayAlarmName, req.RelayFar, 0, towerRelayAlarmReason, nil)
	}
	if req.RelayNear > 0 {
		handleAlarm(ctx, alram, NearRelayAlarmName, req.RelayNear, 0, towerRelayAlarmReason, nil)
	}
	if req.RelayHigh > 0 {
		handleAlarm(ctx, alram, HighRelayAlarmName, req.RelayHigh, 0, towerRelayAlarmReason, nil)
	}
	if req.RelayLow > 0 {
		handleAlarm(ctx, alram, LowRelayAlarmName, req.RelayLow, 0, towerRelayAlarmReason, nil)
	}

	return nil
}

type TowerRealtimeDataRequest struct {
	// sn
	Sn          string `json:"sn"`
	CompanyCode byte   `json:"company_code"`
	// 状态采集时间
	Timestamp int64 `json:"timestamp"`
	// 回转角度值
	BackTurn float64 `json:"back_turn"`
	// 幅度值
	Scope float64 `json:"scope"`
	// 吊钩离地面的距离
	Hight float64 `json:"hight"`
	// 吊钩所掉重物的重量
	Weight float64 `json:"weight"`
	// 当前力矩所占最大力矩的百分比
	Moment int `json:"moment"`
	// 电池电量
	Battery int `json:"battery"`
	// 风速
	WindSpeed float64 `json:"wind_speed"`
	// 塔身倾斜度 X 向
	AngleX float64 `json:"angle_x"`
	// 塔身倾斜度 Y 向
	AngleY float64 `json:"angle_y"`

	/*限位报警编码  0000 无，0001 预警，0010 报警，0100 故障*/
	// 左限位
	PositionLeftAlarm int `json:"position_left_alarm"`
	// 右限位
	PositionRightAlarm int `json:"position_right_alarm"`
	// 远限位
	PositionFarAlarm int `json:"position_far_alarm"`
	// 近限位
	PositionNearAlarm int `json:"position_near_alarm"`
	// 高限位
	PositionHighAlarm int `json:"position_high_alarm"`
	// 低限位
	PositionLowAlarm int `json:"position_low_alarm"`

	// 回转机械左限位 0-正常，1-回转机械左限位故障
	MachineBackTurnLeftAlarm byte `json:"machine_back_turn_left_alarm"`
	// 回转机械右限位 0-正常，1-回转机械右限位故障
	MachineBackTurnRightAlarm byte `json:"machine_back_turn_right_alarm"`
	// 变幅机械远限位 0-正常，1-变幅机械远限位故障
	MachineScopeFarAlarm byte `json:"machine_scope_far_alarm"`
	// 变幅机械近限位 0-正常，1-变幅机械近限位故障
	MachineScopeNearAlarm byte `json:"machine_scope_near_alarm"`
	// 高度机械限位 0-正常，1-高度机械限位故障
	MachineHighAlarm byte `json:"machine_high_alarm"`
	// 起重机械限位 0-正常，1-起重机械限位故障
	MachineWeightAlarm byte `json:"machine_weight_alarm"`
	// 力矩机械限位 0-正常，1-力矩机械限位故障
	MachineMomentAlarm byte `json:"machine_moment_alarm"`

	/*其他报警编码  */
	// 起重 0000 无，0001 重载，0010 超载，0011 违章，0100 故障
	WeightAlarm int `json:"weigh_alarm"`
	// 力矩 0000 无，0001 重载，0010 超载，0011 违章，0100 故障
	MomentAlarm int `json:"moment_alarm"`
	// 风速 0000 无，0001 预警，0010 报警，0100 故障
	WindSpeedAlarm int `json:"wind_speed_alarm"`
	// 塔身倾角 0000 无，0010 报警
	AngleAlarm int `json:"angle_alarm"`

	/*塔机碰撞报警编码  0000 无，0001 预警，0010 报警*/
	// 左碰撞
	LeftCollisionAlarm int `json:"left_collision_alarm"`
	// 右碰撞
	RightCollisionAlarm int `json:"right_collision_alarm"`
	// 远碰撞
	FarCollisionAlarm int `json:"far_collision_alarm"`
	// 近碰撞
	NearCollisionAlarm int `json:"near_collision_alarm"`
	// 低障碍
	LowCollisionAlarm int `json:"low_collision_alarm"`
	// 0-前臂，1-后壁
	CollisionDirection byte `json:"collision_direction"`

	/*禁区碰撞报警编码   0000 无，0001 预警，0010 报警，0011 违章*/
	// 左侧禁行
	ForbiddenLeft int `json:"forbidden_left"`
	// 右侧禁行
	ForbiddenRight int `json:"forbidden_right"`
	// 0-前臂，1-后壁
	ForbiddenDirection byte `json:"forbidden_direction"`

	/*障碍物碰撞报警编码  0000 无，0001 预警，0010 报警*/
	// 左侧障碍
	ObstacleLeftCollisionAlarm int `json:"obstacle_left_collision_alarm"`
	// 右侧障碍
	ObstacleRightCollisionAlarm int `json:"obstacle_right_collision_alarm"`
	// 远方障碍
	ObstacleFarCollisionAlarm int `json:"obstacle_far_collision_alarm"`
	// 近方障碍
	ObstacleNearCollisionAlarm int `json:"obstacle_near_collision_alarm"`
	// 低障碍
	ObstacleLowCollisionAlarm int `json:"obstacle_low_collision_alarm"`
	// 0-前臂，1-后壁
	ObstacleCollisionDirection byte `json:"obstacle_collision_direction"`

	/*继电输出 0000 无，0001 截断低速，0010 截断高速, 如果不支持，可填为全 0*/
	// 左
	RelayLeft int `json:"relay_left"`
	// 右
	RelayRight int `json:"relay_right"`
	// 远
	RelayFar int `json:"relay_far"`
	// 近
	RelayNear int `json:"relay_near"`
	// 高
	RelayHigh int `json:"relay_high"`
	// 低
	RelayLow int `json:"relay_low"`
}

func TowerRealtimeDataWrite(req *TowerRealtimeDataRequest) error {
	tags := map[string]string{"sn": req.Sn}
	fields := map[string]interface{}{}

	bytes, _ := json.Marshal(req)
	json.Unmarshal(bytes, &fields)
	delete(fields, "sn")
	delete(fields, "timestamp")
	t := time.Unix(req.Timestamp, 0)

	// 写数据
	err := model.WriteTowerData(req.Sn, tags, fields, t, fmt.Sprintf("%x", TowerFrameRealtimeDataRequest))
	if err != nil {
		return err
	}
	return nil
}
